"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.V1CredentialService = void 0;
const tsyringe_1 = require("tsyringe");
const AgentConfig_1 = require("../../../../agent/AgentConfig");
const Dispatcher_1 = require("../../../../agent/Dispatcher");
const EventEmitter_1 = require("../../../../agent/EventEmitter");
const Attachment_1 = require("../../../../decorators/attachment/Attachment");
const error_1 = require("../../../../error");
const storage_1 = require("../../../../storage");
const utils_1 = require("../../../../utils");
const attachment_1 = require("../../../../utils/attachment");
const uuid_1 = require("../../../../utils/uuid");
const common_1 = require("../../../common");
const services_1 = require("../../../connections/services");
const routing_1 = require("../../../routing");
const IndyCredentialFormatService_1 = require("../../formats/indy/IndyCredentialFormatService");
const models_1 = require("../../formats/indy/models");
const CredentialAutoAcceptType_1 = require("../../models/CredentialAutoAcceptType");
const CredentialState_1 = require("../../models/CredentialState");
const repository_1 = require("../../repository");
const services_2 = require("../../services");
const composeAutoAccept_1 = require("../../util/composeAutoAccept");
const previewAttributes_1 = require("../../util/previewAttributes");
const handlers_1 = require("./handlers");
const messages_1 = require("./messages");
const V1CredentialPreview_1 = require("./messages/V1CredentialPreview");
let V1CredentialService = class V1CredentialService extends services_2.CredentialService {
    constructor(connectionService, didCommMessageRepository, agentConfig, mediationRecipientService, dispatcher, eventEmitter, credentialRepository, formatService) {
        super(credentialRepository, didCommMessageRepository, eventEmitter, dispatcher, agentConfig);
        /**
         * The version of the issue credential protocol this service supports
         */
        this.version = 'v1';
        this.connectionService = connectionService;
        this.formatService = formatService;
        this.didCommMessageRepository = didCommMessageRepository;
        this.mediationRecipientService = mediationRecipientService;
        this.registerHandlers();
    }
    getFormatServiceForRecordType(credentialRecordType) {
        if (credentialRecordType !== this.formatService.credentialRecordType) {
            throw new error_1.AriesFrameworkError(`Unsupported credential record type ${credentialRecordType} for v1 issue credential protocol (need ${this.formatService.credentialRecordType})`);
        }
        return this.formatService;
    }
    /**
     * Create a {@link ProposeCredentialMessage} not bound to an existing credential exchange.
     * To create a proposal as response to an existing credential exchange, use {@link createProposalAsResponse}.
     *
     * @param options The object containing config options
     * @returns Object containing proposal message and associated credential record
     *
     */
    async createProposal({ connection, credentialFormats, comment, autoAcceptCredential, }) {
        this.assertOnlyIndyFormat(credentialFormats);
        if (!credentialFormats.indy) {
            throw new error_1.AriesFrameworkError('Missing indy credential format in v1 create proposal call.');
        }
        // TODO: linked attachments are broken currently. We never include them in the messages.
        // The linking with previews does work, so it shouldn't be too much work to re-enable this.
        const { linkedAttachments } = credentialFormats.indy;
        // Create record
        const credentialRecord = new repository_1.CredentialExchangeRecord({
            connectionId: connection.id,
            threadId: uuid_1.uuid(),
            state: CredentialState_1.CredentialState.ProposalSent,
            linkedAttachments: linkedAttachments === null || linkedAttachments === void 0 ? void 0 : linkedAttachments.map((linkedAttachment) => linkedAttachment.attachment),
            autoAcceptCredential: autoAcceptCredential,
            protocolVersion: 'v1',
        });
        // call create proposal for validation of the proposal and addition of linked attachments
        const { previewAttributes, attachment } = await this.formatService.createProposal({
            credentialFormats,
            credentialRecord,
        });
        // Transform the attachment into the attachment payload and use that to construct the v1 message
        const indyCredentialProposal = utils_1.JsonTransformer.fromJSON(attachment.getDataAsJson(), models_1.IndyCredPropose);
        const credentialProposal = previewAttributes
            ? new V1CredentialPreview_1.V1CredentialPreview({
                attributes: previewAttributes,
            })
            : undefined;
        // Create message
        const message = new messages_1.V1ProposeCredentialMessage(Object.assign(Object.assign({}, indyCredentialProposal), { id: credentialRecord.threadId, credentialPreview: credentialProposal, comment }));
        await this.didCommMessageRepository.saveAgentMessage({
            agentMessage: message,
            role: storage_1.DidCommMessageRole.Sender,
            associatedRecordId: credentialRecord.id,
        });
        credentialRecord.credentialAttributes = previewAttributes;
        await this.credentialRepository.save(credentialRecord);
        this.emitStateChangedEvent(credentialRecord, null);
        return { credentialRecord, message };
    }
    /**
     * Process a received {@link ProposeCredentialMessage}. This will not accept the credential proposal
     * or send a credential offer. It will only create a new, or update the existing credential record with
     * the information from the credential proposal message. Use {@link createOfferAsResponse}
     * after calling this method to create a credential offer.
     *
     * @param messageContext The message context containing a credential proposal message
     * @returns credential record associated with the credential proposal message
     *
     */
    async processProposal(messageContext) {
        const { message: proposalMessage, connection } = messageContext;
        this.logger.debug(`Processing credential proposal with message id ${proposalMessage.id}`);
        let credentialRecord = await this.findByThreadAndConnectionId(proposalMessage.threadId, connection === null || connection === void 0 ? void 0 : connection.id);
        // Credential record already exists, this is a response to an earlier message sent by us
        if (credentialRecord) {
            this.logger.debug('Credential record already exists for incoming proposal');
            // Assert
            credentialRecord.assertProtocolVersion('v1');
            credentialRecord.assertState(CredentialState_1.CredentialState.OfferSent);
            const proposalCredentialMessage = await this.didCommMessageRepository.findAgentMessage({
                associatedRecordId: credentialRecord.id,
                messageClass: messages_1.V1ProposeCredentialMessage,
            });
            const offerCredentialMessage = await this.didCommMessageRepository.findAgentMessage({
                associatedRecordId: credentialRecord.id,
                messageClass: messages_1.V1OfferCredentialMessage,
            });
            this.connectionService.assertConnectionOrServiceDecorator(messageContext, {
                previousReceivedMessage: proposalCredentialMessage !== null && proposalCredentialMessage !== void 0 ? proposalCredentialMessage : undefined,
                previousSentMessage: offerCredentialMessage !== null && offerCredentialMessage !== void 0 ? offerCredentialMessage : undefined,
            });
            await this.formatService.processProposal({
                credentialRecord,
                attachment: new Attachment_1.Attachment({
                    data: new Attachment_1.AttachmentData({
                        json: utils_1.JsonTransformer.toJSON(this.rfc0592ProposalFromV1ProposeMessage(proposalMessage)),
                    }),
                }),
            });
            // Update record
            await this.updateState(credentialRecord, CredentialState_1.CredentialState.ProposalReceived);
            await this.didCommMessageRepository.saveOrUpdateAgentMessage({
                agentMessage: proposalMessage,
                role: storage_1.DidCommMessageRole.Receiver,
                associatedRecordId: credentialRecord.id,
            });
        }
        else {
            this.logger.debug('Credential record does not exists yet for incoming proposal');
            // No credential record exists with thread id
            credentialRecord = new repository_1.CredentialExchangeRecord({
                connectionId: connection === null || connection === void 0 ? void 0 : connection.id,
                threadId: proposalMessage.threadId,
                state: CredentialState_1.CredentialState.ProposalReceived,
                protocolVersion: 'v1',
            });
            // Assert
            this.connectionService.assertConnectionOrServiceDecorator(messageContext);
            // Save record
            await this.credentialRepository.save(credentialRecord);
            this.emitStateChangedEvent(credentialRecord, null);
            await this.didCommMessageRepository.saveAgentMessage({
                agentMessage: proposalMessage,
                role: storage_1.DidCommMessageRole.Receiver,
                associatedRecordId: credentialRecord.id,
            });
        }
        return credentialRecord;
    }
    /**
     * Processing an incoming credential message and create a credential offer as a response
     * @param options The object containing config options
     * @returns Object containing proposal message and associated credential record
     */
    async acceptProposal({ credentialRecord, credentialFormats, comment, autoAcceptCredential, }) {
        var _a;
        // Assert
        credentialRecord.assertProtocolVersion('v1');
        credentialRecord.assertState(CredentialState_1.CredentialState.ProposalReceived);
        if (credentialFormats)
            this.assertOnlyIndyFormat(credentialFormats);
        const proposalMessage = await this.didCommMessageRepository.getAgentMessage({
            associatedRecordId: credentialRecord.id,
            messageClass: messages_1.V1ProposeCredentialMessage,
        });
        // NOTE: We set the credential attributes from the proposal on the record as we've 'accepted' them
        // and can now use them to create the offer in the format services. It may be overwritten later on
        // if the user provided other attributes in the credentialFormats array.
        credentialRecord.credentialAttributes = (_a = proposalMessage.credentialPreview) === null || _a === void 0 ? void 0 : _a.attributes;
        const { attachment, previewAttributes } = await this.formatService.acceptProposal({
            attachId: messages_1.INDY_CREDENTIAL_OFFER_ATTACHMENT_ID,
            credentialFormats,
            credentialRecord,
            proposalAttachment: new Attachment_1.Attachment({
                data: new Attachment_1.AttachmentData({
                    json: utils_1.JsonTransformer.toJSON(this.rfc0592ProposalFromV1ProposeMessage(proposalMessage)),
                }),
            }),
        });
        if (!previewAttributes) {
            throw new error_1.AriesFrameworkError('Missing required credential preview attributes from indy format service');
        }
        const message = new messages_1.V1OfferCredentialMessage({
            comment,
            offerAttachments: [attachment],
            credentialPreview: new V1CredentialPreview_1.V1CredentialPreview({
                attributes: previewAttributes,
            }),
            attachments: credentialRecord.linkedAttachments,
        });
        message.setThread({ threadId: credentialRecord.threadId });
        credentialRecord.credentialAttributes = previewAttributes;
        credentialRecord.autoAcceptCredential = autoAcceptCredential !== null && autoAcceptCredential !== void 0 ? autoAcceptCredential : credentialRecord.autoAcceptCredential;
        await this.updateState(credentialRecord, CredentialState_1.CredentialState.OfferSent);
        await this.didCommMessageRepository.saveOrUpdateAgentMessage({
            agentMessage: message,
            role: storage_1.DidCommMessageRole.Sender,
            associatedRecordId: credentialRecord.id,
        });
        return { credentialRecord, message };
    }
    /**
     * Negotiate a credential proposal as issuer (by sending a credential offer message) to the connection
     * associated with the credential record.
     *
     * @param options configuration for the offer see {@link NegotiateProposalOptions}
     * @returns Credential record associated with the credential offer and the corresponding new offer message
     *
     */
    async negotiateProposal({ credentialFormats, credentialRecord, comment, autoAcceptCredential, }) {
        // Assert
        credentialRecord.assertProtocolVersion('v1');
        credentialRecord.assertState(CredentialState_1.CredentialState.ProposalReceived);
        if (credentialFormats)
            this.assertOnlyIndyFormat(credentialFormats);
        const { attachment, previewAttributes } = await this.formatService.createOffer({
            attachId: messages_1.INDY_CREDENTIAL_OFFER_ATTACHMENT_ID,
            credentialFormats,
            credentialRecord,
        });
        if (!previewAttributes) {
            throw new error_1.AriesFrameworkError('Missing required credential preview attributes from indy format service');
        }
        const message = new messages_1.V1OfferCredentialMessage({
            comment,
            offerAttachments: [attachment],
            credentialPreview: new V1CredentialPreview_1.V1CredentialPreview({
                attributes: previewAttributes,
            }),
            attachments: credentialRecord.linkedAttachments,
        });
        message.setThread({ threadId: credentialRecord.threadId });
        credentialRecord.credentialAttributes = previewAttributes;
        credentialRecord.autoAcceptCredential = autoAcceptCredential !== null && autoAcceptCredential !== void 0 ? autoAcceptCredential : credentialRecord.autoAcceptCredential;
        await this.updateState(credentialRecord, CredentialState_1.CredentialState.OfferSent);
        await this.didCommMessageRepository.saveOrUpdateAgentMessage({
            agentMessage: message,
            role: storage_1.DidCommMessageRole.Sender,
            associatedRecordId: credentialRecord.id,
        });
        return { credentialRecord, message };
    }
    /**
     * Create a {@link OfferCredentialMessage} not bound to an existing credential exchange.
     * To create an offer as response to an existing credential exchange, use {@link V1CredentialService#createOfferAsResponse}.
     *
     * @param options The options containing config params for creating the credential offer
     * @returns Object containing offer message and associated credential record
     *
     */
    async createOffer({ credentialFormats, autoAcceptCredential, comment, connection, }) {
        var _a, _b;
        // Assert
        if (credentialFormats)
            this.assertOnlyIndyFormat(credentialFormats);
        if (!credentialFormats.indy) {
            throw new error_1.AriesFrameworkError('Missing indy credential format data for v1 create offer');
        }
        // Create record
        const credentialRecord = new repository_1.CredentialExchangeRecord({
            connectionId: connection === null || connection === void 0 ? void 0 : connection.id,
            threadId: uuid_1.uuid(),
            linkedAttachments: (_a = credentialFormats.indy.linkedAttachments) === null || _a === void 0 ? void 0 : _a.map((linkedAttachments) => linkedAttachments.attachment),
            state: CredentialState_1.CredentialState.OfferSent,
            autoAcceptCredential,
            protocolVersion: 'v1',
        });
        const { attachment, previewAttributes } = await this.formatService.createOffer({
            attachId: messages_1.INDY_CREDENTIAL_OFFER_ATTACHMENT_ID,
            credentialFormats,
            credentialRecord,
        });
        if (!previewAttributes) {
            throw new error_1.AriesFrameworkError('Missing required credential preview from indy format service');
        }
        // Construct offer message
        const message = new messages_1.V1OfferCredentialMessage({
            id: credentialRecord.threadId,
            credentialPreview: new V1CredentialPreview_1.V1CredentialPreview({
                attributes: previewAttributes,
            }),
            comment,
            offerAttachments: [attachment],
            attachments: (_b = credentialFormats.indy.linkedAttachments) === null || _b === void 0 ? void 0 : _b.map((linkedAttachments) => linkedAttachments.attachment),
        });
        await this.didCommMessageRepository.saveAgentMessage({
            associatedRecordId: credentialRecord.id,
            agentMessage: message,
            role: storage_1.DidCommMessageRole.Sender,
        });
        credentialRecord.credentialAttributes = previewAttributes;
        await this.credentialRepository.save(credentialRecord);
        this.emitStateChangedEvent(credentialRecord, null);
        return { message, credentialRecord };
    }
    /**
     * Process a received {@link OfferCredentialMessage}. This will not accept the credential offer
     * or send a credential request. It will only create a new credential record with
     * the information from the credential offer message. Use {@link createRequest}
     * after calling this method to create a credential request.
     *
     * @param messageContext The message context containing a credential request message
     * @returns credential record associated with the credential offer message
     *
     */
    async processOffer(messageContext) {
        const { message: offerMessage, connection } = messageContext;
        this.logger.debug(`Processing credential offer with id ${offerMessage.id}`);
        let credentialRecord = await this.findByThreadAndConnectionId(offerMessage.threadId, connection === null || connection === void 0 ? void 0 : connection.id);
        const offerAttachment = offerMessage.getOfferAttachmentById(messages_1.INDY_CREDENTIAL_OFFER_ATTACHMENT_ID);
        if (!offerAttachment) {
            throw new error_1.AriesFrameworkError(`Indy attachment with id ${messages_1.INDY_CREDENTIAL_OFFER_ATTACHMENT_ID} not found in offer message`);
        }
        if (credentialRecord) {
            const proposalCredentialMessage = await this.didCommMessageRepository.findAgentMessage({
                associatedRecordId: credentialRecord.id,
                messageClass: messages_1.V1ProposeCredentialMessage,
            });
            const offerCredentialMessage = await this.didCommMessageRepository.findAgentMessage({
                associatedRecordId: credentialRecord.id,
                messageClass: messages_1.V1OfferCredentialMessage,
            });
            // Assert
            credentialRecord.assertProtocolVersion('v1');
            credentialRecord.assertState(CredentialState_1.CredentialState.ProposalSent);
            this.connectionService.assertConnectionOrServiceDecorator(messageContext, {
                previousReceivedMessage: offerCredentialMessage !== null && offerCredentialMessage !== void 0 ? offerCredentialMessage : undefined,
                previousSentMessage: proposalCredentialMessage !== null && proposalCredentialMessage !== void 0 ? proposalCredentialMessage : undefined,
            });
            await this.formatService.processOffer({
                credentialRecord,
                attachment: offerAttachment,
            });
            await this.didCommMessageRepository.saveOrUpdateAgentMessage({
                agentMessage: offerMessage,
                role: storage_1.DidCommMessageRole.Receiver,
                associatedRecordId: credentialRecord.id,
            });
            await this.updateState(credentialRecord, CredentialState_1.CredentialState.OfferReceived);
            return credentialRecord;
        }
        else {
            // No credential record exists with thread id
            credentialRecord = new repository_1.CredentialExchangeRecord({
                connectionId: connection === null || connection === void 0 ? void 0 : connection.id,
                threadId: offerMessage.id,
                state: CredentialState_1.CredentialState.OfferReceived,
                protocolVersion: 'v1',
            });
            // Assert
            this.connectionService.assertConnectionOrServiceDecorator(messageContext);
            await this.formatService.processOffer({
                credentialRecord,
                attachment: offerAttachment,
            });
            // Save in repository
            await this.didCommMessageRepository.saveAgentMessage({
                agentMessage: offerMessage,
                role: storage_1.DidCommMessageRole.Receiver,
                associatedRecordId: credentialRecord.id,
            });
            await this.credentialRepository.save(credentialRecord);
            this.emitStateChangedEvent(credentialRecord, null);
            return credentialRecord;
        }
    }
    /**
     * Create a {@link RequestCredentialMessage} as response to a received credential offer.
     *
     * @param options configuration to use for the credential request
     * @returns Object containing request message and associated credential record
     *
     */
    async acceptOffer({ credentialRecord, credentialFormats, comment, autoAcceptCredential, }) {
        var _a, _b;
        // Assert credential
        credentialRecord.assertProtocolVersion('v1');
        credentialRecord.assertState(CredentialState_1.CredentialState.OfferReceived);
        const offerMessage = await this.didCommMessageRepository.getAgentMessage({
            associatedRecordId: credentialRecord.id,
            messageClass: messages_1.V1OfferCredentialMessage,
        });
        const offerAttachment = offerMessage.getOfferAttachmentById(messages_1.INDY_CREDENTIAL_OFFER_ATTACHMENT_ID);
        if (!offerAttachment) {
            throw new error_1.AriesFrameworkError(`Indy attachment with id ${messages_1.INDY_CREDENTIAL_OFFER_ATTACHMENT_ID} not found in offer message`);
        }
        const { attachment } = await this.formatService.acceptOffer({
            credentialRecord,
            credentialFormats,
            attachId: messages_1.INDY_CREDENTIAL_REQUEST_ATTACHMENT_ID,
            offerAttachment,
        });
        const requestMessage = new messages_1.V1RequestCredentialMessage({
            comment,
            requestAttachments: [attachment],
            attachments: (_a = offerMessage.appendedAttachments) === null || _a === void 0 ? void 0 : _a.filter((attachment) => attachment_1.isLinkedAttachment(attachment)),
        });
        requestMessage.setThread({ threadId: credentialRecord.threadId });
        credentialRecord.credentialAttributes = offerMessage.credentialPreview.attributes;
        credentialRecord.autoAcceptCredential = autoAcceptCredential !== null && autoAcceptCredential !== void 0 ? autoAcceptCredential : credentialRecord.autoAcceptCredential;
        credentialRecord.linkedAttachments = (_b = offerMessage.appendedAttachments) === null || _b === void 0 ? void 0 : _b.filter((attachment) => attachment_1.isLinkedAttachment(attachment));
        await this.didCommMessageRepository.saveOrUpdateAgentMessage({
            agentMessage: requestMessage,
            associatedRecordId: credentialRecord.id,
            role: storage_1.DidCommMessageRole.Sender,
        });
        await this.updateState(credentialRecord, CredentialState_1.CredentialState.RequestSent);
        return { message: requestMessage, credentialRecord };
    }
    /**
     * Process a received {@link RequestCredentialMessage}. This will not accept the credential request
     * or send a credential. It will only update the existing credential record with
     * the information from the credential request message. Use {@link createCredential}
     * after calling this method to create a credential.
     *
     * @param messageContext The message context containing a credential request message
     * @returns credential record associated with the credential request message
     *
     */
    async negotiateOffer({ credentialFormats, credentialRecord, autoAcceptCredential, comment, }) {
        // Assert
        credentialRecord.assertProtocolVersion('v1');
        credentialRecord.assertState(CredentialState_1.CredentialState.OfferReceived);
        this.assertOnlyIndyFormat(credentialFormats);
        if (!credentialRecord.connectionId) {
            throw new error_1.AriesFrameworkError(`No connectionId found for credential record '${credentialRecord.id}'. Connection-less issuance does not support negotiation.`);
        }
        if (!credentialFormats.indy) {
            throw new error_1.AriesFrameworkError('Missing indy credential format in v1 create proposal call.');
        }
        const { linkedAttachments } = credentialFormats.indy;
        // call create proposal for validation of the proposal and addition of linked attachments
        // As the format is different for v1 of the issue credential protocol we won't be using the attachment
        const { previewAttributes, attachment } = await this.formatService.createProposal({
            credentialFormats,
            credentialRecord,
        });
        // Transform the attachment into the attachment payload and use that to construct the v1 message
        const indyCredentialProposal = utils_1.JsonTransformer.fromJSON(attachment.getDataAsJson(), models_1.IndyCredPropose);
        const credentialProposal = previewAttributes
            ? new V1CredentialPreview_1.V1CredentialPreview({
                attributes: previewAttributes,
            })
            : undefined;
        // Create message
        const message = new messages_1.V1ProposeCredentialMessage(Object.assign(Object.assign({}, indyCredentialProposal), { credentialPreview: credentialProposal, comment }));
        message.setThread({ threadId: credentialRecord.threadId });
        await this.didCommMessageRepository.saveOrUpdateAgentMessage({
            agentMessage: message,
            role: storage_1.DidCommMessageRole.Sender,
            associatedRecordId: credentialRecord.id,
        });
        // Update record
        credentialRecord.credentialAttributes = previewAttributes;
        credentialRecord.linkedAttachments = linkedAttachments === null || linkedAttachments === void 0 ? void 0 : linkedAttachments.map((attachment) => attachment.attachment);
        credentialRecord.autoAcceptCredential = autoAcceptCredential !== null && autoAcceptCredential !== void 0 ? autoAcceptCredential : credentialRecord.autoAcceptCredential;
        await this.updateState(credentialRecord, CredentialState_1.CredentialState.ProposalSent);
        return { credentialRecord, message };
    }
    /**
     * Starting from a request is not supported in v1 of the issue credential protocol
     * because indy doesn't allow to start from a request
     */
    async createRequest() {
        throw new error_1.AriesFrameworkError('Starting from a request is not supported for v1 issue credential protocol');
    }
    /**
     * Process a received {@link IssueCredentialMessage}. This will not accept the credential
     * or send a credential acknowledgement. It will only update the existing credential record with
     * the information from the issue credential message. Use {@link createAck}
     * after calling this method to create a credential acknowledgement.
     *
     * @param messageContext The message context containing an issue credential message
     *
     * @returns credential record associated with the issue credential message
     *
     */
    async processRequest(messageContext) {
        const { message: requestMessage, connection } = messageContext;
        this.logger.debug(`Processing credential request with id ${requestMessage.id}`);
        const credentialRecord = await this.getByThreadAndConnectionId(requestMessage.threadId, connection === null || connection === void 0 ? void 0 : connection.id);
        this.logger.trace('Credential record found when processing credential request', credentialRecord);
        const proposalMessage = await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: credentialRecord.id,
            messageClass: messages_1.V1ProposeCredentialMessage,
        });
        const offerMessage = await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: credentialRecord.id,
            messageClass: messages_1.V1OfferCredentialMessage,
        });
        // Assert
        credentialRecord.assertProtocolVersion('v1');
        credentialRecord.assertState(CredentialState_1.CredentialState.OfferSent);
        this.connectionService.assertConnectionOrServiceDecorator(messageContext, {
            previousReceivedMessage: proposalMessage !== null && proposalMessage !== void 0 ? proposalMessage : undefined,
            previousSentMessage: offerMessage !== null && offerMessage !== void 0 ? offerMessage : undefined,
        });
        const requestAttachment = requestMessage.getRequestAttachmentById(messages_1.INDY_CREDENTIAL_REQUEST_ATTACHMENT_ID);
        if (!requestAttachment) {
            throw new error_1.AriesFrameworkError(`Indy attachment with id ${messages_1.INDY_CREDENTIAL_REQUEST_ATTACHMENT_ID} not found in request message`);
        }
        await this.formatService.processRequest({
            credentialRecord,
            attachment: requestAttachment,
        });
        await this.didCommMessageRepository.saveAgentMessage({
            agentMessage: requestMessage,
            role: storage_1.DidCommMessageRole.Receiver,
            associatedRecordId: credentialRecord.id,
        });
        await this.updateState(credentialRecord, CredentialState_1.CredentialState.RequestReceived);
        return credentialRecord;
    }
    /**
     * Create a {@link IssueCredentialMessage} as response to a received credential request.
     *
     * @returns Object containing issue credential message and associated credential record
     *
     */
    async acceptRequest({ credentialRecord, credentialFormats, comment, autoAcceptCredential, }) {
        // Assert
        credentialRecord.assertProtocolVersion('v1');
        credentialRecord.assertState(CredentialState_1.CredentialState.RequestReceived);
        const offerMessage = await this.didCommMessageRepository.getAgentMessage({
            associatedRecordId: credentialRecord.id,
            messageClass: messages_1.V1OfferCredentialMessage,
        });
        const requestMessage = await this.didCommMessageRepository.getAgentMessage({
            associatedRecordId: credentialRecord.id,
            messageClass: messages_1.V1RequestCredentialMessage,
        });
        const offerAttachment = offerMessage.getOfferAttachmentById(messages_1.INDY_CREDENTIAL_OFFER_ATTACHMENT_ID);
        const requestAttachment = requestMessage.getRequestAttachmentById(messages_1.INDY_CREDENTIAL_REQUEST_ATTACHMENT_ID);
        if (!offerAttachment || !requestAttachment) {
            throw new error_1.AriesFrameworkError(`Missing data payload in offer or request attachment in credential Record ${credentialRecord.id}`);
        }
        const { attachment: credentialsAttach } = await this.formatService.acceptRequest({
            credentialRecord,
            requestAttachment,
            offerAttachment,
            attachId: messages_1.INDY_CREDENTIAL_ATTACHMENT_ID,
            credentialFormats,
        });
        const issueMessage = new messages_1.V1IssueCredentialMessage({
            comment,
            credentialAttachments: [credentialsAttach],
            attachments: credentialRecord.linkedAttachments,
        });
        issueMessage.setThread({ threadId: credentialRecord.threadId });
        issueMessage.setPleaseAck();
        await this.didCommMessageRepository.saveAgentMessage({
            agentMessage: issueMessage,
            associatedRecordId: credentialRecord.id,
            role: storage_1.DidCommMessageRole.Sender,
        });
        credentialRecord.autoAcceptCredential = autoAcceptCredential !== null && autoAcceptCredential !== void 0 ? autoAcceptCredential : credentialRecord.autoAcceptCredential;
        await this.updateState(credentialRecord, CredentialState_1.CredentialState.CredentialIssued);
        return { message: issueMessage, credentialRecord };
    }
    /**
     * Process an incoming {@link IssueCredentialMessage}
     *
     * @param messageContext The message context containing a credential acknowledgement message
     * @returns credential record associated with the credential acknowledgement message
     *
     */
    async processCredential(messageContext) {
        const { message: issueMessage, connection } = messageContext;
        this.logger.debug(`Processing credential with id ${issueMessage.id}`);
        const credentialRecord = await this.getByThreadAndConnectionId(issueMessage.threadId, connection === null || connection === void 0 ? void 0 : connection.id);
        const requestCredentialMessage = await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: credentialRecord.id,
            messageClass: messages_1.V1RequestCredentialMessage,
        });
        const offerCredentialMessage = await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: credentialRecord.id,
            messageClass: messages_1.V1OfferCredentialMessage,
        });
        // Assert
        credentialRecord.assertProtocolVersion('v1');
        credentialRecord.assertState(CredentialState_1.CredentialState.RequestSent);
        this.connectionService.assertConnectionOrServiceDecorator(messageContext, {
            previousReceivedMessage: offerCredentialMessage !== null && offerCredentialMessage !== void 0 ? offerCredentialMessage : undefined,
            previousSentMessage: requestCredentialMessage !== null && requestCredentialMessage !== void 0 ? requestCredentialMessage : undefined,
        });
        const issueAttachment = issueMessage.getCredentialAttachmentById(messages_1.INDY_CREDENTIAL_ATTACHMENT_ID);
        if (!issueAttachment) {
            throw new error_1.AriesFrameworkError('Missing indy credential attachment in processCredential');
        }
        await this.formatService.processCredential({
            attachment: issueAttachment,
            credentialRecord,
        });
        await this.didCommMessageRepository.saveAgentMessage({
            agentMessage: issueMessage,
            role: storage_1.DidCommMessageRole.Receiver,
            associatedRecordId: credentialRecord.id,
        });
        await this.updateState(credentialRecord, CredentialState_1.CredentialState.CredentialReceived);
        return credentialRecord;
    }
    /**
     * Create a {@link CredentialAckMessage} as response to a received credential.
     *
     * @param credentialRecord The credential record for which to create the credential acknowledgement
     * @returns Object containing credential acknowledgement message and associated credential record
     *
     */
    async acceptCredential({ credentialRecord, }) {
        credentialRecord.assertProtocolVersion('v1');
        credentialRecord.assertState(CredentialState_1.CredentialState.CredentialReceived);
        // Create message
        const ackMessage = new messages_1.V1CredentialAckMessage({
            status: common_1.AckStatus.OK,
            threadId: credentialRecord.threadId,
        });
        await this.updateState(credentialRecord, CredentialState_1.CredentialState.Done);
        return { message: ackMessage, credentialRecord };
    }
    /**
     * Process a received {@link CredentialAckMessage}.
     *
     * @param messageContext The message context containing a credential acknowledgement message
     * @returns credential record associated with the credential acknowledgement message
     *
     */
    async processAck(messageContext) {
        const { message: ackMessage, connection } = messageContext;
        this.logger.debug(`Processing credential ack with id ${ackMessage.id}`);
        const credentialRecord = await this.getByThreadAndConnectionId(ackMessage.threadId, connection === null || connection === void 0 ? void 0 : connection.id);
        const requestCredentialMessage = await this.didCommMessageRepository.getAgentMessage({
            associatedRecordId: credentialRecord.id,
            messageClass: messages_1.V1RequestCredentialMessage,
        });
        const issueCredentialMessage = await this.didCommMessageRepository.getAgentMessage({
            associatedRecordId: credentialRecord.id,
            messageClass: messages_1.V1IssueCredentialMessage,
        });
        // Assert
        credentialRecord.assertProtocolVersion('v1');
        credentialRecord.assertState(CredentialState_1.CredentialState.CredentialIssued);
        this.connectionService.assertConnectionOrServiceDecorator(messageContext, {
            previousReceivedMessage: requestCredentialMessage,
            previousSentMessage: issueCredentialMessage,
        });
        // Update record
        await this.updateState(credentialRecord, CredentialState_1.CredentialState.Done);
        return credentialRecord;
    }
    // AUTO RESPOND METHODS
    async shouldAutoRespondToProposal(options) {
        const { credentialRecord, proposalMessage } = options;
        const autoAccept = composeAutoAccept_1.composeAutoAccept(credentialRecord.autoAcceptCredential, this.agentConfig.autoAcceptCredentials);
        // Handle always / never cases
        if (autoAccept === CredentialAutoAcceptType_1.AutoAcceptCredential.Always)
            return true;
        if (autoAccept === CredentialAutoAcceptType_1.AutoAcceptCredential.Never)
            return false;
        const offerMessage = await this.findOfferMessage(credentialRecord.id);
        // Do not auto accept if missing properties
        if (!offerMessage || !offerMessage.credentialPreview)
            return false;
        if (!proposalMessage.credentialPreview || !proposalMessage.credentialDefinitionId)
            return false;
        const credentialOfferJson = offerMessage.indyCredentialOffer;
        // Check if credential definition id matches
        if (!credentialOfferJson)
            return false;
        if (credentialOfferJson.cred_def_id !== proposalMessage.credentialDefinitionId)
            return false;
        // Check if preview values match
        return previewAttributes_1.arePreviewAttributesEqual(proposalMessage.credentialPreview.attributes, offerMessage.credentialPreview.attributes);
    }
    async shouldAutoRespondToOffer(options) {
        const { credentialRecord, offerMessage } = options;
        const autoAccept = composeAutoAccept_1.composeAutoAccept(credentialRecord.autoAcceptCredential, this.agentConfig.autoAcceptCredentials);
        // Handle always / never cases
        if (autoAccept === CredentialAutoAcceptType_1.AutoAcceptCredential.Always)
            return true;
        if (autoAccept === CredentialAutoAcceptType_1.AutoAcceptCredential.Never)
            return false;
        const proposalMessage = await this.findProposalMessage(credentialRecord.id);
        // Do not auto accept if missing properties
        if (!offerMessage.credentialPreview)
            return false;
        if (!proposalMessage || !proposalMessage.credentialPreview || !proposalMessage.credentialDefinitionId)
            return false;
        const credentialOfferJson = offerMessage.indyCredentialOffer;
        // Check if credential definition id matches
        if (!credentialOfferJson)
            return false;
        if (credentialOfferJson.cred_def_id !== proposalMessage.credentialDefinitionId)
            return false;
        // Check if preview values match
        return previewAttributes_1.arePreviewAttributesEqual(proposalMessage.credentialPreview.attributes, offerMessage.credentialPreview.attributes);
    }
    async shouldAutoRespondToRequest(options) {
        const { credentialRecord, requestMessage } = options;
        const autoAccept = composeAutoAccept_1.composeAutoAccept(credentialRecord.autoAcceptCredential, this.agentConfig.autoAcceptCredentials);
        // Handle always / never cases
        if (autoAccept === CredentialAutoAcceptType_1.AutoAcceptCredential.Always)
            return true;
        if (autoAccept === CredentialAutoAcceptType_1.AutoAcceptCredential.Never)
            return false;
        const offerMessage = await this.findOfferMessage(credentialRecord.id);
        if (!offerMessage)
            return false;
        const offerAttachment = offerMessage.getOfferAttachmentById(messages_1.INDY_CREDENTIAL_OFFER_ATTACHMENT_ID);
        const requestAttachment = requestMessage.getRequestAttachmentById(messages_1.INDY_CREDENTIAL_REQUEST_ATTACHMENT_ID);
        if (!offerAttachment || !requestAttachment)
            return false;
        return this.formatService.shouldAutoRespondToRequest({
            credentialRecord,
            offerAttachment,
            requestAttachment,
        });
    }
    async shouldAutoRespondToCredential(options) {
        const { credentialRecord, credentialMessage } = options;
        const autoAccept = composeAutoAccept_1.composeAutoAccept(credentialRecord.autoAcceptCredential, this.agentConfig.autoAcceptCredentials);
        // Handle always / never cases
        if (autoAccept === CredentialAutoAcceptType_1.AutoAcceptCredential.Always)
            return true;
        if (autoAccept === CredentialAutoAcceptType_1.AutoAcceptCredential.Never)
            return false;
        const requestMessage = await this.findRequestMessage(credentialRecord.id);
        const offerMessage = await this.findOfferMessage(credentialRecord.id);
        const credentialAttachment = credentialMessage.getCredentialAttachmentById(messages_1.INDY_CREDENTIAL_ATTACHMENT_ID);
        if (!credentialAttachment)
            return false;
        const requestAttachment = requestMessage === null || requestMessage === void 0 ? void 0 : requestMessage.getRequestAttachmentById(messages_1.INDY_CREDENTIAL_REQUEST_ATTACHMENT_ID);
        if (!requestAttachment)
            return false;
        const offerAttachment = offerMessage === null || offerMessage === void 0 ? void 0 : offerMessage.getOfferAttachmentById(messages_1.INDY_CREDENTIAL_OFFER_ATTACHMENT_ID);
        return this.formatService.shouldAutoRespondToCredential({
            credentialRecord,
            credentialAttachment,
            requestAttachment,
            offerAttachment,
        });
    }
    async findProposalMessage(credentialExchangeId) {
        return await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: credentialExchangeId,
            messageClass: messages_1.V1ProposeCredentialMessage,
        });
    }
    async findOfferMessage(credentialExchangeId) {
        return await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: credentialExchangeId,
            messageClass: messages_1.V1OfferCredentialMessage,
        });
    }
    async findRequestMessage(credentialExchangeId) {
        return await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: credentialExchangeId,
            messageClass: messages_1.V1RequestCredentialMessage,
        });
    }
    async findCredentialMessage(credentialExchangeId) {
        return await this.didCommMessageRepository.findAgentMessage({
            associatedRecordId: credentialExchangeId,
            messageClass: messages_1.V1IssueCredentialMessage,
        });
    }
    async getFormatData(credentialExchangeId) {
        var _a, _b, _c, _d, _e;
        // TODO: we could looking at fetching all record using a single query and then filtering based on the type of the message.
        const [proposalMessage, offerMessage, requestMessage, credentialMessage] = await Promise.all([
            this.findProposalMessage(credentialExchangeId),
            this.findOfferMessage(credentialExchangeId),
            this.findRequestMessage(credentialExchangeId),
            this.findCredentialMessage(credentialExchangeId),
        ]);
        const indyProposal = proposalMessage
            ? utils_1.JsonTransformer.toJSON(this.rfc0592ProposalFromV1ProposeMessage(proposalMessage))
            : undefined;
        const indyOffer = (_a = offerMessage === null || offerMessage === void 0 ? void 0 : offerMessage.indyCredentialOffer) !== null && _a !== void 0 ? _a : undefined;
        const indyRequest = (_b = requestMessage === null || requestMessage === void 0 ? void 0 : requestMessage.indyCredentialRequest) !== null && _b !== void 0 ? _b : undefined;
        const indyCredential = (_c = credentialMessage === null || credentialMessage === void 0 ? void 0 : credentialMessage.indyCredential) !== null && _c !== void 0 ? _c : undefined;
        return {
            proposalAttributes: (_d = proposalMessage === null || proposalMessage === void 0 ? void 0 : proposalMessage.credentialPreview) === null || _d === void 0 ? void 0 : _d.attributes,
            proposal: proposalMessage
                ? {
                    indy: indyProposal,
                }
                : undefined,
            offerAttributes: (_e = offerMessage === null || offerMessage === void 0 ? void 0 : offerMessage.credentialPreview) === null || _e === void 0 ? void 0 : _e.attributes,
            offer: offerMessage
                ? {
                    indy: indyOffer,
                }
                : undefined,
            request: requestMessage
                ? {
                    indy: indyRequest,
                }
                : undefined,
            credential: credentialMessage
                ? {
                    indy: indyCredential,
                }
                : undefined,
        };
    }
    registerHandlers() {
        this.dispatcher.registerHandler(new handlers_1.V1ProposeCredentialHandler(this, this.agentConfig));
        this.dispatcher.registerHandler(new handlers_1.V1OfferCredentialHandler(this, this.agentConfig, this.mediationRecipientService, this.didCommMessageRepository));
        this.dispatcher.registerHandler(new handlers_1.V1RequestCredentialHandler(this, this.agentConfig, this.didCommMessageRepository));
        this.dispatcher.registerHandler(new handlers_1.V1IssueCredentialHandler(this, this.agentConfig, this.didCommMessageRepository));
        this.dispatcher.registerHandler(new handlers_1.V1CredentialAckHandler(this));
        this.dispatcher.registerHandler(new handlers_1.V1CredentialProblemReportHandler(this));
    }
    rfc0592ProposalFromV1ProposeMessage(proposalMessage) {
        const indyCredentialProposal = new models_1.IndyCredPropose({
            credentialDefinitionId: proposalMessage.credentialDefinitionId,
            schemaId: proposalMessage.schemaId,
            issuerDid: proposalMessage.issuerDid,
            schemaIssuerDid: proposalMessage.schemaIssuerDid,
            schemaName: proposalMessage.schemaName,
            schemaVersion: proposalMessage.schemaVersion,
        });
        return indyCredentialProposal;
    }
    assertOnlyIndyFormat(credentialFormats) {
        const formatKeys = Object.keys(credentialFormats);
        // It's fine to not have any formats in some cases, if indy is required the method that calls this should check for this
        if (formatKeys.length === 0)
            return;
        if (formatKeys.length !== 1 || !formatKeys.includes('indy')) {
            throw new error_1.AriesFrameworkError('Only indy credential format is supported for issue credential v1 protocol');
        }
    }
};
V1CredentialService = __decorate([
    tsyringe_1.scoped(tsyringe_1.Lifecycle.ContainerScoped),
    __metadata("design:paramtypes", [services_1.ConnectionService,
        storage_1.DidCommMessageRepository,
        AgentConfig_1.AgentConfig,
        routing_1.MediationRecipientService,
        Dispatcher_1.Dispatcher,
        EventEmitter_1.EventEmitter,
        repository_1.CredentialRepository,
        IndyCredentialFormatService_1.IndyCredentialFormatService])
], V1CredentialService);
exports.V1CredentialService = V1CredentialService;
//# sourceMappingURL=V1CredentialService.js.map